package org.livee.data.batch;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.db.AggregationCommand;
import org.livee.data.db.CommandFactory;
import org.livee.data.db.DatabaseAdapter;

import org.livee.data.tools.LogManager;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * 
 */
public class AggregationBatch implements AbstractBatch {
	
	int m_Interval;
	
	public AggregationBatch(){		
		org.livee.data.tools.ConfigManager config=org.livee.data.tools.ConfigManager.getInstance();
		m_Interval=config.getIntParameter("/c:liveeConfiguration/c:master/c:aggregator/c:interval/text()");
		if(m_Interval<=0){
			m_Interval=300;
		}
	}
	
	public int interval(){
		return m_Interval;
	}
	
	public void execute() throws Exception {

		LogManager.postNormal("master.aggregator.begin", null);
		
		try{
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			AggregationCommand command=CommandFactory.createAggregationCommand();
		
			boolean tryNext=true;
			while(tryNext){
				db.postCommand(command);
				if(command.succeeded()){
					tryNext=command.tryNext();
				}else{
					// Error
					LogManager.postError("master.aggregator.error", "Aggregation Failed..");
				}
			}
		}
		catch(Exception e){
			LogManager.postError("master.aggregator.error", e.getMessage());
			return ;
		}
		LogManager.postNormal("master.aggregator.end", null);
	}

	public String getBatchName() {
		return "Aggregation";
	}

}
