package org.livee.data.batch;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.db.CommandFactory;
import org.livee.data.db.DatabaseAdapter;
import org.livee.data.db.RestoreAdminCommand;
import org.livee.data.expr.AbstractData;
import org.livee.data.expr.AbstractParser;
import org.livee.data.expr.ParserFactory;
import org.livee.util.Xml;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.livee.data.tools.LogManager;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * 
 */
public class CopyAdminBatch implements AbstractBatch {

	int m_Interval;
	String m_MasterAdminURL;
	
	public CopyAdminBatch(){		
		org.livee.data.tools.ConfigManager config=org.livee.data.tools.ConfigManager.getInstance();
		m_Interval=config.getIntParameter("/c:liveeConfiguration/c:slave/c:copyAdmin/c:interval/text()");
		m_MasterAdminURL=config.getStringParameter("/c:liveeConfiguration/c:slave/c:masterAdminURL/text()");
		if(m_Interval<=0){
			m_Interval=600;
		}
	}
	
	public int interval(){
		return m_Interval;
	}
	
	public void execute() throws Exception {

		LogManager.postNormal("slave.copyAdmin.begin", null);
		
		if(m_MasterAdminURL==null){
			// Error
			LogManager.postError("slave.copyAdmin.error", "MasterAdminURL Error");
			return ;
		}
		
		String replicated_xml=null;
		try{
			org.livee.data.soap.client.Admin200703Service srv=new org.livee.data.soap.client.Admin200703ServiceLocator();
			org.livee.data.soap.client.Admin200703_PortType admin=srv.getAdmin200703(new java.net.URL(m_MasterAdminURL));			
			replicated_xml=admin.replicate();
		}
		catch(Exception e){
			// e.printStackTrace();
			LogManager.postError("slave.copyAdmin.error", e.getMessage());
			return ;
		}
			
		if(replicated_xml==null){
			// Error throw new Exception("Failed to get master's replication..");
			LogManager.postError("slave.copyAdmin.error", "Failed to get master's replication file.");
			return ;
		}
			
		try{
			Document doc=Xml.parseXml(replicated_xml);
			Element root=doc.getDocumentElement();
							
			ParserFactory pf=ParserFactory.getInstance();
			AbstractParser adminReplicate_parser=pf.get("adminReplicate");
			AbstractData adminReplicate=adminReplicate_parser.parse(root);
				
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			RestoreAdminCommand command=CommandFactory.createRestoreAdminCommand(adminReplicate);
			db.postCommand(command);
				
			if(!command.succeeded()){
				// Error 	throw new Exception("Failed to load master's replication..");
				LogManager.postError("slave.copyAdmin.error","Failed to parse or save master's replication file.");
				return ;
			}

		}catch(Exception e){
			// Error
			LogManager.postNormal("slave.copyAdmin.error", e.getMessage());
			return ;
		}

		LogManager.postNormal("slave.copyAdmin.end", null);
	}

	public String getBatchName() {
		return "CopyAdmin";
	}

}
