package org.livee.data.batch;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.util.Calendar;

import org.livee.data.db.DatabaseAdapter;
import org.livee.data.expr.AbstractData;
import org.livee.data.expr.AbstractParser;
import org.livee.util.W3CTimestamp;
import org.livee.util.Xml;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.livee.data.tools.LogManager;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * 
 */
public class CopyArchiveDataBatch implements AbstractBatch {

	int m_Interval;
	String m_MasterURL;
	String m_MasterAdminURL;
	String m_DefaultStart;
	int m_ThresholdTime;
	String m_ThresholdTimeBase;
	int m_ScopeTime;
	String m_ScopeTimeBase;
	
	public CopyArchiveDataBatch(){		
		org.livee.data.tools.ConfigManager config=org.livee.data.tools.ConfigManager.getInstance();
		m_MasterURL=config.getStringParameter("/c:liveeConfiguration/c:slave/c:copyDataManager/c:masterURL/text()");
		m_MasterAdminURL=config.getStringParameter("/c:liveeConfiguration/c:slave/c:masterAdminURL/text()");

		m_Interval=config.getIntParameter("/c:liveeConfiguration/c:slave/c:copyAdmin/c:interval/text()");
		m_DefaultStart=config.getStringParameter("/c:liveeConfiguration/c:slave/c:copyDataManager/c:archiveDataUpdate/c:start/text()");
		m_ThresholdTime=config.getIntParameter("/c:liveeConfiguration/c:slave/c:copyDataManager/c:archiveDataUpdate/c:thresholdTime/text()");
		m_ThresholdTimeBase=config.getStringParameter("/c:liveeConfiguration/c:slave/c:copyDataManager/c:archiveDataUpdate/c:thresholdTimeBase/text()");
		m_ScopeTime=config.getIntParameter("/c:liveeConfiguration/c:slave/c:copyDataManager/c:archiveDataUpdate/c:scopeTime/text()");
		m_ScopeTimeBase=config.getStringParameter("/c:liveeConfiguration/c:slave/c:copyDataManager/c:archiveDataUpdate/c:scopeTimeBase/text()");
		
		if(m_Interval<=0){
			m_Interval=600;
		}
		
		if(m_ThresholdTime<0){
			m_ThresholdTime=600;
		}
		if(m_ThresholdTimeBase==null || !m_ThresholdTimeBase.matches("second|minute|hour|day|month")){
			m_ThresholdTimeBase="second";
		}
		
		if(m_ScopeTime<0){
			m_ScopeTime=600;
		}
		if(m_ScopeTimeBase==null || !m_ScopeTimeBase.matches("second|minute|hour|day|month")){
			m_ScopeTimeBase="second";
		}
	}
	
	public int interval(){
		return m_Interval;
	}
	public void execute() throws Exception {
	
		LogManager.postNormal("slave.copyDataManager.archiveDataUpdate.begin", null);
		
		boolean bType=true;
		try{
			java.util.Calendar comparator=null;
			java.util.Calendar from=null;
			java.util.Calendar to=null;
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			
			
			while(bType){
				org.livee.data.replicate.GetSlaveReplicationProfileCommand gscommand=new org.livee.data.replicate.GetSlaveReplicationProfileCommand("dataArchive");
				db.postCommand(gscommand);
				
				if(gscommand.succeeded()){
					from=gscommand.getDone();
				}else{
					from=W3CTimestamp.parse(m_DefaultStart);
				}
				comparator=java.util.Calendar.getInstance();
				if(m_ThresholdTimeBase.equals("second")){
					comparator.add(Calendar.SECOND, -m_ThresholdTime);
				}else if(m_ThresholdTimeBase.equals("minute")){
					comparator.add(Calendar.MINUTE, -m_ThresholdTime);				
				}else if(m_ThresholdTimeBase.equals("hour")){
					comparator.add(Calendar.HOUR, -m_ThresholdTime);				
				}else if(m_ThresholdTimeBase.equals("day")){
					comparator.add(Calendar.DATE, -m_ThresholdTime);				
				}else if(m_ThresholdTimeBase.equals("month")){
					comparator.add(Calendar.MONTH, -m_ThresholdTime);				
				}
				if(from.after(comparator)){
					return ;
				}
				to=java.util.Calendar.getInstance();
				to.setTimeInMillis(from.getTimeInMillis());
				if(m_ScopeTimeBase.equals("second")){
					to.add(Calendar.SECOND, m_ScopeTime);
				}else if(m_ScopeTimeBase.equals("minute")){
					to.add(Calendar.MINUTE, m_ScopeTime);
				}else if(m_ScopeTimeBase.equals("hour")){
					to.add(Calendar.HOUR, m_ScopeTime);
				}else if(m_ScopeTimeBase.equals("day")){
					to.add(Calendar.DATE, m_ScopeTime);
				}else if(m_ScopeTimeBase.equals("month")){
					to.add(Calendar.MONTH, m_ScopeTime);
				}
				
				LogManager.postNormal("slave.copyDataManager.archiveDataUpdate.message", "Updating rawArchive at "+W3CTimestamp.toString(from));
				
				// DataManagerReplicationService Location Resolution
				try{
					org.livee.data.soap.client.Admin200703Service admins=new org.livee.data.soap.client.Admin200703ServiceLocator();
					org.livee.data.soap.client.Admin200703_PortType admin=admins.getAdmin200703(new java.net.URL(m_MasterAdminURL));
					
					String masterURL=admin.queryService("DataManagerReplication200703");
					if(masterURL!=null && masterURL.startsWith("http://")){
						m_MasterURL=masterURL;
					}
				}
				catch(Exception e){
					e.printStackTrace();
				}
				
				
				// Replication start
				org.livee.data.soap.client.DataManagerReplicationService drps=new org.livee.data.soap.client.DataManagerReplicationServiceLocator();
				org.livee.data.soap.client.DataManagerReplication srv=drps.getDataManagerReplication200703(new java.net.URL(m_MasterURL));
				
				String dataManager_string=srv.replicateArchiveData(W3CTimestamp.toString(from), W3CTimestamp.toString(to));
				// System.out.println(dataManager_string);
				
				Document doc=Xml.parseXml(dataManager_string);
				Element dataManager_xml=doc.getDocumentElement();
				
				AbstractParser dm_parser=org.livee.data.replicate.ParserFactory.getInstance().get("replicateDataManager");
				AbstractData dataManager=dm_parser.parse(dataManager_xml);
							
				org.livee.data.replicate.RestoreArchiveDataCommand command=new org.livee.data.replicate.RestoreArchiveDataCommand(dataManager);
				db.postCommand(command);
				
				if(command.succeeded()){
					org.livee.data.replicate.SetSlaveReplicationProfileCommand sscommand=new org.livee.data.replicate.SetSlaveReplicationProfileCommand("dataArchive",to);
					db.postCommand(sscommand);
					
					if(sscommand.succeeded()){
						LogManager.postNormal("slave.copyDataManager.archiveDataUpdate.message", "Successfully loaded..");
					}else{
						LogManager.postError("slave.copyDataManager.archiveDataUpdate.error","Failed to update \"slave replication profile\".");
						bType=false;
					}
				}else{
					LogManager.postError("slave.copyDataManager.archiveDataUpdate.error","Failed to save replicated raw archive data.");
					bType=false;
				}
			}
		}
		catch(Exception e){
			// e.printStackTrace();
			LogManager.postError("slave.copyDataManager.archiveDataUpdate.error",e.getMessage());
		}
		
		LogManager.postNormal("slave.copyDataManager.archiveDataUpdate.end", null);
	}

	public String getBatchName() {
		return "CopyArchiveData";
	}

}
