package org.livee.data.batch;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.db.DatabaseAdapter;
import org.livee.data.expr.AbstractData;
import org.livee.data.expr.AbstractParser;
import org.livee.util.Xml;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.livee.data.tools.LogManager;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * 
 */
public class CopyLatestDataBatch implements AbstractBatch {

	int m_Interval;
	String m_MasterURL;
	String m_MasterAdminURL;
	
	public CopyLatestDataBatch(){		
		org.livee.data.tools.ConfigManager config=org.livee.data.tools.ConfigManager.getInstance();
		m_Interval=config.getIntParameter("/c:liveeConfiguration/c:slave/c:copyDataManager/c:latestDataUpdate/c:interval/text()");
		m_MasterURL=config.getStringParameter("/c:liveeConfiguration/c:slave/c:copyDataManager/c:masterURL/text()");
		m_MasterAdminURL=config.getStringParameter("/c:liveeConfiguration/c:slave/c:masterAdminURL/text()");

		if(m_Interval<=0){
			m_Interval=60;
		}
	}
	public int interval(){
		return m_Interval;
	}
	
	public void execute() throws Exception {

		LogManager.postNormal("slave.copyDataManager.latestDataUpdate.begin", null);
		
		try{
			
			// DataManagerReplicationService Location Resolution
			try{
				org.livee.data.soap.client.Admin200703Service admins=new org.livee.data.soap.client.Admin200703ServiceLocator();
				org.livee.data.soap.client.Admin200703_PortType admin=admins.getAdmin200703(new java.net.URL(m_MasterAdminURL));
				
				String masterURL=admin.queryService("DataManagerReplication200703");
				if(masterURL!=null && masterURL.startsWith("http://")){
					m_MasterURL=masterURL;
				}
			}
			catch(Exception e){
				e.printStackTrace();
			}
			
			
			// Replication start
			org.livee.data.soap.client.DataManagerReplicationService drps=new org.livee.data.soap.client.DataManagerReplicationServiceLocator();
			org.livee.data.soap.client.DataManagerReplication srv=drps.getDataManagerReplication200703(new java.net.URL(m_MasterURL));
			
			String dataManager_string=srv.replicateLatestData();
			
			Document doc=Xml.parseXml(dataManager_string);
			Element dataManager_xml=doc.getDocumentElement();
			
			AbstractParser dm_parser=org.livee.data.replicate.ParserFactory.getInstance().get("replicateDataManager");
			AbstractData dataManager=dm_parser.parse(dataManager_xml);
			
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			
			org.livee.data.replicate.RestoreLatestDataCommand command=new org.livee.data.replicate.RestoreLatestDataCommand(dataManager);
			db.postCommand(command);
			
			if(!command.succeeded()){
				LogManager.postError("slave.copyDataManager.latestDataUpdate.error", "Failed to copy master's latest data.");
			}			
		}
		catch(Exception e){
			// e.printStackTrace();
			LogManager.postError("slave.copyDataManager.latestDataUpdate.error", e.getMessage());
		}
		
		LogManager.postNormal("slave.copyDataManager.latestDataUpdate.end", null);
	}

	public String getBatchName() {
		return "CopyLatestData";
	}

}
