package org.livee.data.command;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Calendar;
import java.util.Map;

import org.livee.data.db.IDbCommand;
import org.livee.data.db.DatabaseAdapter;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * 
 */
public class CSVArchiver {
	
	public class RawDataAbstractionCommand implements IDbCommand {

		boolean m_bSucceeded;
		String m_ID;
		String m_From;
		String m_To;
		public RawDataAbstractionCommand(String id,String from,String to){
			m_ID=id;
			m_From=from;
			m_To=to;
			m_bSucceeded=false;
		}
		
		public boolean succeeded(){
			return m_bSucceeded;
		}
		
		public void execCommand(Connection arg0, Map arg1) throws Exception {

			java.sql.Statement stmt=null;
			
			try{
				stmt=arg0.createStatement();
				
				String sql="SELECT id FROM sensor.element WHERE id=\'"+m_ID+"\';";
				java.sql.ResultSet rset=stmt.executeQuery(sql);
				if(!rset.next()){
					rset.close();
					throw new Exception("ElementSensor: \'"+m_ID+"\' not found.");
				}
				rset.close();
				
				StringBuilder query=new StringBuilder("SELECT id,time,attrs,value,sValue FROM raw.archive WHERE id='"+m_ID+"\'");
				if(m_From!=null){
					query.append(" AND time>=\'");
					query.append(m_From);
					query.append("\'");
				}
				if(m_To!=null){
					query.append(" AND time<\'");
					query.append(m_To);
					query.append("\'");
				}
				query.append(" ORDER BY time");
				
				String declare="DECLARE mycur CURSOR FOR "+query.toString();

				System.out.println("# "+m_ID);
				
				stmt.execute("START TRANSACTION;");
				stmt.execute(declare);
				
				while(true){
					rset=stmt.executeQuery("FETCH NEXT FROM mycur;");
					if(rset.next()){
						String id=rset.getString("id");
						String time=org.livee.util.W3CTimestamp.toString(rset.getTimestamp("time"), java.util.TimeZone.getDefault());
						String attrs=rset.getString("attrs");
						String value=rset.getString("value");
						String sValue=rset.getString("sValue");
						
						//System.out.print("\""+id+"\",");
						System.out.print("\""+time+"\",");
						System.out.print("\""+attrs+"\",");
						System.out.print(value+",");
						System.out.println("\""+sValue+"\"");
					}else{
						rset.close();
						break;
					}
					rset.close();
				}
				m_bSucceeded=true;
			}
			catch(Exception e){
				throw e;
			}
			finally{
				stmt.close();
			}
			
		}

		public String getCommandName() {
			return "RawDataAbstraction";
		}

		public short getImplVersion() {
			return 0x0100;
		}

	}
	
	String m_ID;
	String m_Start;
	String m_End;
	
	public boolean parseArgs(String[] args){
		boolean bRet=false;
		if(args.length>=1){
			m_ID=args[0];
			int i=1;
			while(i<args.length-1){
				if(args[i].equals("-s")){
					m_Start=args[i+1];
				}else if(args[i].equals("-e")){
					m_End=args[i+1];
				}
				i+=2;
			}
			bRet=true;
		}
		
		if(!bRet){
			printUsage();
		}
		
		return bRet;
	}
	
	private void printUsage(){
		System.out.println("Usage: java org.livee.data.command.CSVArchiver ElementID [OPTIONS]");
		System.out.println();
		System.out.println("OPTIONS:=(OPTION)* | e");
		System.out.println("OPTION:= -s START_TIMESTAMP | -e END_TIMESTAMP");
		System.out.println("E.g. -s 2007-01-01T00:00:00+09 ");
	}


	private void execute(String[] args) throws Exception {
		if(parseArgs(args)){
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			RawDataAbstractionCommand command=new RawDataAbstractionCommand(m_ID,m_Start,m_End);
			
			db.postCommand(command);
		}
	}
	
	
	public static void main(String[] args) throws Exception {
		CSVArchiver archiver=new CSVArchiver();
		archiver.execute(args);
	}
	
}
