package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.expr.AbstractData;
import org.livee.data.expr.DataFactory;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * 
 */
public class GetAdminProfileCommand implements IDbCommand {
	
	private boolean m_Succeeded;
	private AbstractData m_Collection;
	
	public GetAdminProfileCommand(){
		m_Succeeded=false;
	}
	
	public boolean succeeded(){
		return m_Succeeded;
	}
	
	public AbstractData getCollection(){
		return m_Collection;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
	
		DataFactory df=DataFactory.getInstance();
		m_Collection=df.create("collection");
		
		java.sql.Statement stmt=null;
		
		try{
			String sql;
			stmt=connection.createStatement();
				
			java.util.Map combinedMap=new java.util.Hashtable();
			sql="SELECT id, name, sValue, fValue, iValue, tValue FROM admin.combined_index ORDER BY id";
			String id=null;
			AbstractData combined=null;
			java.sql.ResultSet rset=stmt.executeQuery(sql);
			while(rset.next()){
				String cid=rset.getString("id");
				String name=rset.getString("name");
				if(!cid.equals(id)){
					id=cid;
					combined=df.create("combined");
					combined.setAttribute("id", id);
					combinedMap.put(id, combined);
					m_Collection.append(combined);
				}
				
				String value=null;
				if(value==null){
					value=rset.getString("sValue");
				}
				if(value==null){
					value=rset.getString("fValue");
				}
				if(value==null){
					value=rset.getString("iValue");
				}
				if(value==null){
					value=rset.getString("tValue");
				}
				
				if(value!=null){
					combined.setAttribute(name, value);
				}	
			}
			rset.close();
			
			java.util.Map elementMap=new java.util.Hashtable();
			sql="SELECT id,name,sValue,fValue,iValue,tValue FROM admin.element_index ORDER BY id;";	
			id=null;
			AbstractData element=null;
			rset=stmt.executeQuery(sql);
			while(rset.next()){
				String cid=rset.getString("id");
				String name=rset.getString("name");
				if(!cid.equals(id)){
					id=cid;
					element=df.create("element");
					element.setAttribute("id", id);
					elementMap.put(id, element);
				}
				
				String value=null;
				if(value==null){
					value=rset.getString("sValue");
				}
				if(value==null){
					value=rset.getString("fValue");
				}
				if(value==null){
					value=rset.getString("iValue");
				}
				if(value==null){
					value=rset.getString("tValue");
				}
				
				if(value!=null){
					element.setAttribute(name, value);
				}	
			}
			rset.close();
			
			sql="SELECT id,combined_id FROM admin.element ORDER BY combined_id;";
			rset=stmt.executeQuery(sql);
			id=null;
			combined=null;
			while(rset.next()){
				String combined_id=rset.getString("combined_id");
				String element_id=rset.getString("id");
				
				if(!combined_id.equals(id)){
					id=combined_id;
					combined=(AbstractData)combinedMap.get(id);
				}
				element=(AbstractData)elementMap.get(element_id);
				if(element!=null && combined!=null){
					combined.append(element);
				}
			}
			rset.close();
			m_Succeeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}

	}

	public String getCommandName() {
		return "GetAdminProfile";
	}

	public short getImplVersion() {
		return 0x0100;
	}


}
