package org.livee.data.db;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.expr.AbstractData;

/**
 * GetElementProfileCommand is obsolete
 * @author Hideya Ochiai, Akihiro Sugiyama, The University of Tokyo
 * @see GetProfileCommand
 */
public class GetElementProfileCommand implements IDbCommand {
	
	/**
	 * Request
	 */
	String m_ID;

	/**
	 * Response
	 */

	AbstractData m_ElementSensor = null;

	private GetElementProfileCommand() {
	}

	protected GetElementProfileCommand(String id) {
		m_ID = id;
		m_Succeeded = false;
	}

	boolean m_Succeeded;

	public boolean succeeded() {
		return m_Succeeded;
	}

	public AbstractData getResult() {
		return m_ElementSensor;
	}

	public String getCommandName() {
		return "GetElementProfile";
	}

	public short getImplVersion() {
		return 0x0100;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {
		
		java.sql.Statement stmt=null;
		m_Succeeded = false;

		try {	
			org.livee.data.expr.DataFactory df = org.livee.data.expr.DataFactory.getInstance();
			stmt = connection.createStatement();
			
			String sql = "SELECT created, sensorType, attrs from profile.element WHERE id=\'" + m_ID + "\';";
			//sql = "SELECT id,created,sensorType,attrs FROM profile.element WHERE combined_id='"
			//		+ m_ID + "';";
			java.sql.ResultSet rset = stmt.executeQuery(sql);
			if (rset.next()) {
				m_ElementSensor = df.create("element");
				m_ElementSensor.setAttribute("id", m_ID);
				m_ElementSensor.setAttribute("created", rset.getString("created"));
				m_ElementSensor.setAttribute("sensorType", rset.getString("sensorType"));
				String attrs = rset.getString("attrs");
				try {
					org.json.JSONObject jo = new org.json.JSONObject(attrs);
					java.util.Iterator itr = jo.keys();
					while (itr.hasNext()) {
						String key = (String) itr.next();
						m_ElementSensor.setAttribute(key, jo.getString(key));
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
				m_Succeeded = true;
			}
		} catch (Exception e) {
			throw e;
		}
		finally{
			stmt.close();
		}
	}

}
