package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.expr.AbstractData;
import org.json.JSONObject;
import org.json.JSONArray;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * 
 */
public class SetAdminNeighborCommand implements IDbCommand {

	private boolean m_Succeeded;
	private AbstractData m_AdminNeighbor;
	
	public SetAdminNeighborCommand(AbstractData adminNeighbor){
		m_AdminNeighbor=adminNeighbor;
		m_Succeeded=false;
	}
	
	public boolean succeeded(){
		return m_Succeeded;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {
		
		String sql;
		java.sql.Statement stmt=null;

		try{
			stmt = connection.createStatement();
			stmt.execute("START TRANSACTION;");
			stmt.execute("DELETE FROM admin.neighbor;");

			java.util.Iterator itr=m_AdminNeighbor.getChildren().iterator();
			while(itr.hasNext()){
				AbstractData adminSite=(AbstractData)itr.next();
				String name=adminSite.getAttribute("name");
				String type=adminSite.getAttribute("type");
				
				String server=null;
				JSONArray json_server=new JSONArray();
				java.util.Iterator itr2=adminSite.getChildren().iterator();
				while(itr2.hasNext()){
					AbstractData adminServer=(AbstractData)itr2.next();
					String service=adminServer.getAttribute("service");
					String url=adminServer.getAttribute("url");
					String linkStatus=adminServer.getAttribute("linkStatus");
					JSONObject map=new JSONObject();
					map.put("service", service);
					map.put("url",url);
					map.put("linkStatus",linkStatus);
					json_server.put(map);
				}
				server=json_server.toString();
				
				
				if(name==null || type==null || server==null){
					continue;
				}
				
				StringBuilder sb=new StringBuilder("INSERT INTO admin.neighbor (name,type,server) VALUES (\'");
				
				sb.append(name);
				sb.append("\',\'");
				sb.append(type);
				sb.append("\',\'");
				sb.append(server);
				sb.append("\');");

				sql=sb.toString();
				
				//System.out.println();
				
				stmt.execute(sql);
			}
			stmt.execute("COMMIT;");
			
			m_Succeeded=true;
		}
		catch(Exception e){
			stmt.execute("ROLLBACK;");
			throw e;
		}
		finally{
			stmt.close();
		}
	}

	public String getCommandName() {
		return "SetAdminNeighbor";
	}

	public short getImplVersion() {
		return 0x0100;
	}


}
