package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;
import java.util.*;

import org.livee.data.expr.AbstractData;
import org.json.*;


/**
 * SetCombinedProfileCommand is obsolete
 * @author Hideya Ochiai, Akihiro Sugiyama, The University of Tokyo
 * @see SetProfileCommand
 */
public class SetCombinedProfileCommand implements IDbCommand {

	/**
	 * Request
	 */
	AbstractData m_CombinedSensor = null;

	/**
	 * Response
	 */
	boolean m_Succeeded;

	private SetCombinedProfileCommand() {
	}

	protected SetCombinedProfileCommand(AbstractData CombinedSensor) {
		m_CombinedSensor = CombinedSensor;
		m_Succeeded = false;
	}

	public String getCommandName() {
		return "SetCombinedProfileCommand";
	}

	public short getImplVersion() {
		return 0x0100;
	}

	public boolean succeeded() {
		return m_Succeeded;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {
		
		java.sql.Statement stmt = null;
		
		try {
			stmt = connection.createStatement();
			String sql;

			String id = null;
			String latitude = null;
			String longitude = null;
			String created = null;
			JSONObject attrs = new JSONObject();

			// Attribute Iteration
			Map map = m_CombinedSensor.getAttributes();
			Iterator itr = map.keySet().iterator();
			while (itr.hasNext()) {
				String key = (String) itr.next();
				String value = (String) map.get(key);

				if (key.equals("latitude")) {
					latitude = value;
				} else if (key.equals("longitude")) {
					longitude = value;
				} else if (key.equals("created")) {
					created = value;
				} else if (key.equals("id")) {
					id = value;
				} else {
					attrs.put(key, value);
				}
			}

			if (created == null){
				created = java.util.Calendar.getInstance().getTime().toString();
			}
			//System.out.println(created);

			stmt.execute("START TRANSACTION;");
			sql = "UPDATE profile.combined_archive SET expired=\'" + created
					+ "\' WHERE id=\'" + id
					+ "\' and expired IS NULL;";
			// debug
			//System.out.println(sql);
			stmt.executeUpdate(sql);
			/*if (stmt.executeUpdate(sql) == 0) {
				stmt.execute("COMMIT;");
				stmt.close();
				m_Succeeded = false;
				return;
			}*/
			
			
			sql = "INSERT INTO profile.combined_archive (id, created, latitude,longitude,attrs) VALUES (\'"
					+ id
					+ "\', \'"
					+ created
					+ "\', "
					+ latitude
					+ ", "
					+ longitude + ", \'" + attrs + "\');";

			// debug
			// System.out.println(sql);
		
			stmt.execute(sql);
			stmt.execute("COMMIT;");
			m_Succeeded = true;

		} catch (Exception e) {
			stmt.execute("ROLLBACK;");
			throw e;
		}
		finally{
			stmt.close();
		}
	}
}
