package org.livee.data.db;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;
import org.livee.util.*;

import org.livee.data.exception.AuthenticationLiveEException;
import org.livee.data.exception.SensorNotFoundLiveEException;

/**
 * @author Hideya Ochiai, Akihiro Sugiyama, The University of Tokyo
 * 
 */
public class UpdatePasswordCommand implements IDbCommand {


	/**
	 * Request
	 */
	String m_ID;
	String m_new_passwd;
	
	/**
	 * Response
	 */
	boolean m_Succeeded;
	String m_old_passwd;

	public boolean succeeded(){
		return m_Succeeded;
	}
	
	private UpdatePasswordCommand(){}
	
	public UpdatePasswordCommand(String id,String new_passwd){
		m_ID=id;
		try{
			m_new_passwd=Security.SHA1(new_passwd);
			m_old_passwd="force";
		}catch(Exception e){
			e.printStackTrace();
		}
		m_Succeeded=false;
	
	}
	
	public UpdatePasswordCommand(String id, String old_passwd, String new_passwd){
		m_ID=id;
		try{
			m_new_passwd=Security.SHA1(new_passwd);
			m_old_passwd=Security.SHA1(old_passwd);
		}catch(Exception e){
			e.printStackTrace();
		}
		m_Succeeded=false;
	}
	
	
	public String getCommandName() {
		return "UpdatePasswordCommand";
	}

	public short getImplVersion() {
		return 0x0100;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
		
		java.sql.Statement stmt=null;
		m_Succeeded=false;
		
		try{
			stmt=connection.createStatement();
			String sql="SELECT passwd FROM sensor.combined WHERE id='"+m_ID+"';";
			stmt.execute("START TRANSACTION;");
			java.sql.ResultSet rset=stmt.executeQuery(sql);
			
			String old_passwd;
			
			if(rset.next()){
				old_passwd=rset.getString("passwd");
			}else{
				// Error not found..
				throw new SensorNotFoundLiveEException(m_ID);
			}
			rset.close();
			
			if (m_old_passwd.equals(old_passwd) || m_old_passwd.equals("force")) {
				sql = "UPDATE sensor.combined SET passwd=\'" + m_new_passwd + "\' WHERE id=\'" + m_ID + "\';";
				stmt.executeUpdate(sql);
				m_Succeeded=true;
			}else{
				throw new AuthenticationLiveEException(m_ID,m_old_passwd);
			}
			stmt.execute("COMMIT;");
		}
		catch(Exception e){
			stmt.execute("ROLLBACK;");
			throw e;
		}
		finally{
			stmt.close();
		}
		
		
	}

	
}
