package org.livee.data.expr;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.util.Collection;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class AdminSiteData implements AbstractData {

	String m_Name;
	String m_Type;
	
	java.util.Collection m_AdminServerCol;
	
	protected AdminSiteData(){
		m_AdminServerCol=new java.util.ArrayList();
	}

	public void append(AbstractData child) {
		if(child==null){
			return ;
		}
		if(child.getDataType().equals("AdminServer")){
			m_AdminServerCol.add(child);
		}
	}

	public String getAttribute(String name) {
		if(name==null){
			return null;
		}
		if(name.equals("name")){
			return m_Name;
		}else if(name.equals("type")){
			return m_Type;
		}
		return null;
	}

	public Map getAttributes() {
		Map attrs=new java.util.Hashtable();

		if(m_Name!=null){
			attrs.put("name",m_Name);
		}
		if(m_Type!=null){
			attrs.put("type", m_Type);
		}
		
		return attrs;
	}

	public Collection getChildren() {
		return m_AdminServerCol;
	}

	public String getDataType() {
		return "AdminSite";
	}

	public String getTextContent() {
		return null;
	}

	public Element getXml(Document doc) {
		
		String ns="http://live-e.org/Administrator/2007/03/";
		Element site=doc.createElementNS(ns, "site");
				
		if(m_Name!=null){
			site.setAttribute("name",m_Name);
		}
		if(m_Type!=null){
			site.setAttribute("type", m_Type);
		}
		
		java.util.Iterator itr=m_AdminServerCol.iterator();
		while(itr.hasNext()){
			Object obj=itr.next();
			if(obj!=null){
				AbstractData lc=(AbstractData)obj;
				site.appendChild(lc.getXml(doc));
			}
		}

		return site;
		
	}

	public void remove(AbstractData child) {}

	public void removeAttribute(String name) {

		if(name==null){
			return ;
		}
		
		if(name.equals("name")){
			m_Name=null;
		}else if(name.equals("type")){
			m_Type=null;
		}
	}

	public void setAttribute(String name, String value) {
		
		if(name==null){
			return ;
		}
		
		if(name.equals("name")){
			m_Name=value;
		}else if(name.equals("type")){
			m_Type=value;
		}
	}

	public void setTextContent(String text) {
	}


}
