package org.livee.data.expr;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.util.Collection;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class CombinedSensorData implements AbstractData {

	String m_ID;
	java.util.Map m_Attributes;
	java.util.Collection m_ElementSensorCol;
	
	protected CombinedSensorData(){
		m_ID="";
		m_Attributes=new java.util.Hashtable();
		m_ElementSensorCol=new java.util.ArrayList();
	}
	
	public void append(AbstractData child) {
		if(child==null){
			return ;
		}
		if(child.getDataType().equals("ElementSensor")){
			m_ElementSensorCol.add(child);
		}
	}

	public String getAttribute(String name) {
		if(name==null){
			return null;
		}
		
		Object obj=m_Attributes.get(name);
		if(obj!=null){
			return (String)obj;
		}
		return null;
	}

	public Map getAttributes() {
		return m_Attributes;
	}

	public Collection getChildren() {
		return m_ElementSensorCol;
	}

	public String getDataType() {
		return "CombinedSensor";
	}

	public String getTextContent() {
		return null;
	}

	public void remove(AbstractData child) {
		if(child!=null){
			m_ElementSensorCol.remove(child);
		}
	}

	public void removeAttribute(String name) {
		if(name!=null){
			m_Attributes.remove(name);
		}
	}

	public void setAttribute(String name, String value) {
		
		if(name!=null){
			if(name.equals("id")){
				m_ID=value;
			}
			if(value!=null){
				m_Attributes.put(name, value);
			}
		}

	}

	public void setTextContent(String text) {}

	public Element getXml(Document doc) {
		
		String ns="http://live-e.org/DataType/2007/03/";
		Element combined=doc.createElementNS(ns, "sensorGroup");
		combined.setAttribute("class", "combined");
		
		java.util.Iterator itr=m_Attributes.keySet().iterator();
		while(itr.hasNext()){
			Object obj=itr.next();
			if(obj!=null){
				String key=(String)obj;
				String value=(String)m_Attributes.get(key);
				
				combined.setAttribute(key, value);
			}
		}
		
		itr=m_ElementSensorCol.iterator();
		while(itr.hasNext()){
			Object obj=itr.next();
			if(obj!=null){
				AbstractData sensor=(AbstractData)obj;
				combined.appendChild(sensor.getXml(doc));
			}
		}
		
		return combined;
	}

}
