package org.livee.data.expr;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.util.Map;

/**
 * Data Factory (Dynamic Data Instance Creator)
 * @author  Hideya Ochiai, The University of Tokyo
 * @version  1.0
 */
public class DataFactory {

	static DataFactory m_Singleton;
	Map m_DataClassMap;
	
	/**
	 * Create this instance and prepare for Data Instance Generation
	 */
	DataFactory(){
		m_DataClassMap=new java.util.Hashtable();
		
		m_DataClassMap.put("adminReplicate","org.livee.data.expr.AdminReplicateData");
		m_DataClassMap.put("adminSchema","org.livee.data.expr.AdminSchemaData");
		m_DataClassMap.put("adminNeighbor","org.livee.data.expr.AdminNeighborData");
		m_DataClassMap.put("adminSite","org.livee.data.expr.AdminSiteData");
		m_DataClassMap.put("adminServer","org.livee.data.expr.AdminServerData");
		
		m_DataClassMap.put("adminConfig","org.livee.data.expr.AdminConfigData");
		m_DataClassMap.put("adminConfigParameter","org.livee.data.expr.AdminConfigParameterData");
		
		m_DataClassMap.put("warning","org.livee.data.expr.WarningData");
		m_DataClassMap.put("warningCollection","org.livee.data.expr.WarningCollectionData");
				
		m_DataClassMap.put("profileSchema", "org.livee.data.expr.ProfileSchemaData");
		m_DataClassMap.put("schema","org.livee.data.expr.SchemaData");

		m_DataClassMap.put("collection", "org.livee.data.expr.SensorCollectionData");
		m_DataClassMap.put("combined","org.livee.data.expr.CombinedSensorData");
		m_DataClassMap.put("element","org.livee.data.expr.ElementSensorData");
		m_DataClassMap.put("value","org.livee.data.expr.ValueData");
		m_DataClassMap.put("aggValue","org.livee.data.expr.AggValueData");

		m_DataClassMap.put("locale", "org.livee.data.expr.LocaleData");
		m_DataClassMap.put("localeCollection", "org.livee.data.expr.LocaleCollectionData");		

		m_DataClassMap.put("Locale", "org.livee.data.expr.LocaleData");
		m_DataClassMap.put("LocaleCollection", "org.livee.data.expr.LocaleCollectionData");		
		m_DataClassMap.put("TimeZone", "org.livee.data.expr.TimeZoneData");
		m_DataClassMap.put("TimeZoneCollection", "org.livee.data.expr.TimeZoneCollectionData");
		m_DataClassMap.put("Collection", "org.livee.data.expr.SensorCollectionData");
		m_DataClassMap.put("CombinedSensor","org.livee.data.expr.CombinedSensorData");
		m_DataClassMap.put("ElementSensor","org.livee.data.expr.ElementSensorData");
		m_DataClassMap.put("ValueData","org.livee.data.expr.ValueData");
		m_Singleton=this;
	}
	
	/**
	 * @return get the Singleton
	 */
	public static DataFactory getInstance(){
		if(m_Singleton==null){
			new DataFactory();
		}
		return m_Singleton;
	}
	
	/**
	 * @param name Specify Data Type
	 * @return Data Object or null
	 */
	public AbstractData create(String name){
		
		Object className=m_DataClassMap.get(name);
		if(className!=null){
			try{
				Class clazz=Class.forName((String)className);
				return (AbstractData)clazz.newInstance();
			}
			catch(Exception e){
				e.printStackTrace();
			}
		}
		return null;
	}
}
