package org.livee.data.expr;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.util.Map;

/**
 * Dymanic Parser Provider
 * @author  Hideya Ochiai, The University of Tokyo
 * @version  1.0
 */
public class ParserFactory {

	static ParserFactory m_Singleton;
	Map m_ParserMap;
	
	/**
	 * Create this instance and prepare for Parser Provider
	 */
	ParserFactory(){
		m_ParserMap=new java.util.Hashtable();

		m_ParserMap.put("adminReplicate", new AdminReplicateParser());
		m_ParserMap.put("adminSchema", new AdminSchemaParser());

		m_ParserMap.put("adminNeighbor", new AdminNeighborParser());
		m_ParserMap.put("adminSite", new AdminSiteParser());
		m_ParserMap.put("adminServer", new AdminServerParser());
		
		m_ParserMap.put("adminConfig", new AdminConfigParser());
		m_ParserMap.put("adminConfigParameter", new AdminConfigParameterParser());

		m_ParserMap.put("warning", new WarningParser());
		m_ParserMap.put("warningCollection", new WarningCollectionParser());
		
		m_ParserMap.put("localeCollection", new LocaleCollectionParser());
		m_ParserMap.put("locale", new LocaleParser());

		m_ParserMap.put("profileSchema",new ProfileSchemaParser());
		m_ParserMap.put("schema",new SchemaParser());
		
		m_ParserMap.put("collection",new SensorCollectionParser());
		m_ParserMap.put("combined",new CombinedSensorParser());
		m_ParserMap.put("element",new ElementSensorParser());
		m_ParserMap.put("value",new ValueParser());
		m_ParserMap.put("aggValue",new AggValueParser());

		m_ParserMap.put("SensorCollection", new SensorCollectionParser());
		m_ParserMap.put("CombinedSensor",new CombinedSensorParser());
		m_ParserMap.put("ElementSensor",new ElementSensorParser());
		m_ParserMap.put("Value",new ValueParser());
		
		m_ParserMap.put("LocaleCollection", new LocaleCollectionParser());
		m_ParserMap.put("Locale", new LocaleParser());

		
		m_Singleton=this;
	}
	
	/**
	 * @return get the Singleton
	 */
	public static ParserFactory getInstance(){
		if(m_Singleton==null){
			new ParserFactory();
		}
		return m_Singleton;
	}
	
	/**
	 * @param name Specify Parser Type
	 * @return Parser or null
	 */
	public AbstractParser get(String name){
		
		Object o=m_ParserMap.get(name);
		if(o!=null){
			return (AbstractParser)o;
		}
		return null;
	}

}
