package org.livee.data.query;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.util.Map;
import org.livee.data.expr.AbstractData;

/**
 * Data Factory (Dynamic Data Instance Creator) which uses org.livee.data.expr.DataFactory
 * @author  Hideya Ochiai, The University of Tokyo
 * @version  1.0
 */
public class DataFactory {

	static DataFactory m_Singleton;
	Map m_DataClassMap;
	
	/**
	 * Create this instance and prepare for Data Instance Generation
	 */
	DataFactory(){
		m_DataClassMap=new java.util.Hashtable();
		
		m_DataClassMap.put("query","org.livee.data.query.QueryData");
		m_DataClassMap.put("filter","org.livee.data.query.FilterData");		
		m_DataClassMap.put("queryResponse","org.livee.data.query.QueryResponseData");
		m_DataClassMap.put("redirectTo","org.livee.data.query.RedirectToData");
		m_Singleton=this;
	}
	
	/**
	 * @return get the Singleton
	 */
	public static DataFactory getInstance(){
		if(m_Singleton==null){
			new DataFactory();
		}
		return m_Singleton;
	}
	
	/**
	 * @param name Specify Data Type
	 * @return Data Object or null
	 */
	public AbstractData create(String name){
		
		Object className=m_DataClassMap.get(name);
		if(className!=null){
			try{
				Class clazz=Class.forName((String)className);
				return (AbstractData)clazz.newInstance();
			}
			catch(Exception e){
				e.printStackTrace();
			}
		}
		return org.livee.data.expr.DataFactory.getInstance().create(name);
	}
}
