package org.livee.data.query;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.util.Map;

import org.livee.data.expr.AbstractData;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class QueryResponseData implements AbstractData {

	String m_Name;
	String m_Domain;
	
	java.util.ArrayList m_AdminSiteCol;
	AbstractData m_RedirectTo;
	
	protected QueryResponseData(){
		m_Name=null;
		m_Domain=null;
		
		m_AdminSiteCol=new java.util.ArrayList();
		m_RedirectTo=null;
	}

	public void append(AbstractData child) {
		if(child==null){
			return ;
		}
		if(child.getDataType().equals("AdminSite")){
			m_AdminSiteCol.add(child);
		}
		if(child.getDataType().equals("RedirectTo")){
			m_RedirectTo=child;
		}
	}

	public String getAttribute(String name) {
		if(name==null){
			return null;
		}
		if(name.equals("name")){
			return m_Name;
		}else if(name.equals("domain")){
			return m_Domain;
		}
		return null;
	}

	public Map getAttributes() {
		Map attrs=new java.util.Hashtable();

		if(m_Name!=null){
			attrs.put("name",m_Name);
		}
		if(m_Domain!=null){
			attrs.put("domain", m_Domain);
		}
		return attrs;
	}

	public java.util.Collection getChildren() {
		java.util.ArrayList array=new java.util.ArrayList();
		array.addAll(m_AdminSiteCol);
		if(m_RedirectTo!=null){
			array.add(m_RedirectTo);
		}
		return array;
	}

	public String getDataType() {
		return "QueryResponse";
	}

	public String getTextContent() {
		return null;
	}

	public Element getXml(Document doc) {
		
		String ns="http://live-e.org/Query/2007/03/";
		Element queryResponse=doc.createElementNS(ns, "queryResponse");
		
		if(m_Name!=null){
			queryResponse.setAttribute("name",m_Name);
		}
		if(m_Domain!=null){
			queryResponse.setAttribute("domain",m_Domain);
		}
		
		if(m_AdminSiteCol!=null){
			int i;
			int size=m_AdminSiteCol.size();
			for(i=0;i<size;i++){
				AbstractData adminSite=(AbstractData)m_AdminSiteCol.get(i);
				queryResponse.appendChild(adminSite.getXml(doc));
			}
		}
		if(m_RedirectTo!=null){
			queryResponse.appendChild(m_RedirectTo.getXml(doc));
		}
		return queryResponse;
	}

	public void remove(AbstractData child) {}

	public void removeAttribute(String name) {

		if(name==null){
			return ;
		}
		
		if(name.equals("name")){
			m_Name=null;
		}else if(name.equals("domain")){
			m_Domain=null;
		}
	}

	public void setAttribute(String name, String value) {
		
		if(name==null){
			return ;
		}
		
		if(name.equals("name")){
			m_Name=value;
		}else if(name.equals("domain")){
			m_Domain=value;
		}
	}

	public void setTextContent(String text) {}

}
