package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.db.IDbCommand;
import org.livee.data.exception.LiveEException;
import org.livee.data.expr.AbstractData;
import org.livee.util.W3CTimestamp;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class GetReplicateRawArchiveCollectionCommand implements IDbCommand {

	String m_Start;
	String m_End;
	AbstractData m_RawArchiveCol;
	boolean m_bSucceeded;
	
	public GetReplicateRawArchiveCollectionCommand(String start,String end){
		m_Start=start;
		m_End=end;
		m_bSucceeded=false;
	}
	
	boolean succeeded(){
		return m_bSucceeded;
	}
	
	public AbstractData getRawArchiveCol(){
		return m_RawArchiveCol;
	}

	
	public void execCommand(Connection connection, Map conf) throws Exception {
	
		java.sql.Statement stmt=null;
		DataFactory df=DataFactory.getInstance();
		m_RawArchiveCol=df.create("replicateRawArchiveCollection");
		
		//java.util.Calendar cal=java.util.Calendar.getInstance();
		//cal.add(java.util.Calendar.DATE, +1);
		//String updated=cal.getTime().toString();

		try{
			stmt=connection.createStatement();
			
			// TODO Support raw.daily
			String sql;
			if(m_Start==null){
				if(m_End==null){
					sql="SELECT id,time,attrs,value,sValue FROM raw.archive;";
				}else{
					sql="SELECT id,time,attrs,value,sValue FROM raw.archive WHERE time<\'"+m_End+"\'";
				}
			}else{
				if(m_End==null){
					sql="SELECT id,time,attrs,value,sValue FROM raw.archive WHERE time>=\'"+m_Start+"\';";
				}else{
					sql="SELECT id,time,attrs,value,sValue FROM raw.archive WHERE time>=\'"+m_Start+"\' AND time<\'"+m_End+"\'";					
				}
			}
			java.sql.ResultSet rset=stmt.executeQuery(sql);
			
			while(rset.next()){
				String id=rset.getString(1);
				String time=W3CTimestamp.toString(rset.getTimestamp(2),java.util.TimeZone.getDefault());
				String attrs=rset.getString(3);
				String value=rset.getString(4);
				String sValue=rset.getString(5);
				
				AbstractData profileCombined=df.create("replicateRawArchive");
				profileCombined.setAttribute("id", id);
				profileCombined.setAttribute("time", time);
				profileCombined.setAttribute("attrs", attrs);
				profileCombined.setAttribute("value", value);
				profileCombined.setAttribute("sValue", sValue);
				
				m_RawArchiveCol.append(profileCombined);
			}
			
			m_bSucceeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}
	}

	public String getCommandName() {
		return "GetReplicateRawArchiveCollection";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
