package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.db.IDbCommand;
import org.livee.data.exception.LiveEException;
import org.livee.data.expr.AbstractData;
import org.livee.data.replicate.DataFactory;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class GetReplicateRawLatestCollectionCommand implements IDbCommand {

	AbstractData m_RawLatestCol;
	boolean m_bSucceeded;
	
	public GetReplicateRawLatestCollectionCommand(){
		m_bSucceeded=false;
	}
	
	boolean succeeded(){
		return m_bSucceeded;
	}
	
	public AbstractData getRawLatestCol(){
		return m_RawLatestCol;
	}

	
	public void execCommand(Connection connection, Map conf) throws Exception {
	
		java.sql.Statement stmt=null;
		DataFactory df=DataFactory.getInstance();
		m_RawLatestCol=df.create("replicateRawLatestCollection");
		try{
			stmt=connection.createStatement();
			String sql="SELECT id,time,attrs,value,sValue FROM raw.latest;";
			java.sql.ResultSet rset=stmt.executeQuery(sql);
			
			while(rset.next()){
				String id=rset.getString(1);
				String time=null;
				java.sql.Timestamp timestamp=rset.getTimestamp(2);
				if(timestamp!=null){
					time=org.livee.util.W3CTimestamp.toString(timestamp, java.util.TimeZone.getDefault());
				}
				String attrs=rset.getString(3);
				String value=rset.getString(4);
				String sValue=rset.getString(5);
				
				AbstractData rawLatest=df.create("replicateRawLatest");
				rawLatest.setAttribute("id", id);
				rawLatest.setAttribute("time", time);
				rawLatest.setAttribute("attrs",attrs);
				rawLatest.setAttribute("value", value);
				rawLatest.setAttribute("sValue", sValue);
				
				m_RawLatestCol.append(rawLatest);
			}
			
			m_bSucceeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}
	}

	public String getCommandName() {
		return "GetReplicateRawLatestCollection";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
