package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.db.IDbCommand;
import org.livee.data.exception.LiveEException;
import org.livee.data.expr.AbstractData;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class GetReplicateSensorElementCollectionCommand implements IDbCommand {

	AbstractData m_SensorElementCol;
	boolean m_bSucceeded;
	
	public GetReplicateSensorElementCollectionCommand(){
		m_bSucceeded=false;
	}
	
	boolean succeeded(){
		return m_bSucceeded;
	}
	
	public AbstractData getSensorElementCol(){
		return m_SensorElementCol;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
	
		java.sql.Statement stmt=null;
		DataFactory df=DataFactory.getInstance();
		m_SensorElementCol=df.create("replicateSensorElementCollection");
		try{
			stmt=connection.createStatement();
			String sql="SELECT id,combined_id,working FROM sensor.element;";
			java.sql.ResultSet rset=stmt.executeQuery(sql);
			
			while(rset.next()){
				String id=rset.getString(1);
				String combined_id=rset.getString(2);
				String working="false";
				if(rset.getBoolean(3)){
					working="true";
				}
				
				AbstractData sensorElement=df.create("replicateSensorElement");
				sensorElement.setAttribute("id", id);
				sensorElement.setAttribute("combined_id", combined_id);
				sensorElement.setAttribute("working",working);
				m_SensorElementCol.append(sensorElement);
			}
			
			m_bSucceeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}
	}


	public String getCommandName() {
		return "GetReplicateSensorElementCollection";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
