package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.db.IDbCommand;
import org.livee.data.expr.AbstractData;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class SetReplicateSensorCombinedCollectionCommand implements IDbCommand {

	AbstractData m_SensorCombinedCol;
	boolean m_bSucceeded;
	
	public SetReplicateSensorCombinedCollectionCommand(AbstractData sensorCombinedCol){
		m_SensorCombinedCol=sensorCombinedCol;
		m_bSucceeded=false;
	}
	
	boolean succeeded(){
		return m_bSucceeded;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
	
		java.sql.Statement stmt=null;
		
		try{
			stmt=connection.createStatement();
			m_bSucceeded=true;
			
			java.util.Iterator itr=m_SensorCombinedCol.getChildren().iterator();
			while(itr.hasNext()){
				AbstractData sensorCombined=(AbstractData)itr.next();
				if(sensorCombined==null){ continue; }
				String id=sensorCombined.getAttribute("id");
				String passwd=sensorCombined.getAttribute("passwd");
				String working=sensorCombined.getAttribute("working");
				String ownerName=sensorCombined.getAttribute("ownerName");
				String ownerMailAddress=sensorCombined.getAttribute("ownerMailAddress");
				
				if(id==null || passwd==null){
					continue;
				}
				
				// Try update first
				StringBuilder update=new StringBuilder("UPDATE sensor.combined SET passwd=\'");
				update.append(passwd);
				update.append("\'");
				
				if(working!=null && working.startsWith("t")){
					update.append(", working=\'t\'");
				}else{
					update.append(", working=\'f\'");
				}

				if(ownerName!=null){
					update.append(", ownerName=\'");
					update.append(ownerName.replaceAll("\'", "\\\\\'"));
					update.append("\'");
				}

				if(ownerMailAddress!=null){
					update.append(", ownerMailAddress=\'");
					update.append(ownerMailAddress.replaceAll("\'", "\\\\\'"));
					update.append("\'");
				}
				
				update.append(" WHERE id=\'");
				update.append(id);
				update.append("\';");
				
				String sql=update.toString();
				
				if(stmt.executeUpdate(sql)==0){
					// No entry to update ==> INSERT action
					StringBuilder insert=new StringBuilder("INSERT INTO sensor.combined (id,passwd,working,ownerName,ownerMailAddress) VALUES (\'");
					insert.append(id);
					insert.append("\',\'");
					insert.append(passwd);
					if(working!=null && working.startsWith("t")){
						insert.append("\',\'t\',");
					}else{
						insert.append("\',\'f\',");
					}
					
					if(ownerName!=null){
						insert.append("\'");
						insert.append(ownerName);
						insert.append("\',");
					}else{
						insert.append("null,");
					}
					if(ownerMailAddress!=null){
						insert.append("\'");
						insert.append(ownerMailAddress);
						insert.append("\');");
					}else{
						insert.append("null);");
					}
					
					sql=insert.toString();
					try{
						stmt.execute(sql);
					}
					catch(Exception e){
						//System.err.println(sql);
						//e.printStackTrace();
						m_bSucceeded=false;
					}
				}
				
				
			}
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}
	}

	public String getCommandName() {
		return "SetReplicateSensorCombinedCollection";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
