package org.livee.data.srv;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.apache.axis.MessageContext;
import org.livee.data.db.CommandFactory;
import org.livee.data.db.DatabaseAdapter;
import org.livee.data.expr.AbstractData;
import org.livee.data.expr.AbstractParser;

import org.livee.util.Xml;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.livee.data.db.GetAdminSchemaCommand;
import org.livee.data.db.GetAdminProfileCommand;
import org.livee.data.db.GetAdminNeighborCommand;
import org.livee.data.db.DumpAdminCommand;

import org.livee.data.tools.ConfigManager;
import org.livee.data.tools.LogManager;
import org.livee.data.exception.LiveEException;
import org.livee.data.exception.InternalLiveEException;
import org.livee.data.exception.NoServerResolvedLiveEException;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class Admin200703 {
	
	public String getProfileAll(){  // Called by parent administrator
		
		LogManager.postNormal("service.Admin200703.getProfileAll.begin", null);
		
		DatabaseAdapter db=DatabaseAdapter.getInstance();
		GetAdminProfileCommand command=CommandFactory.createGetAdminProfileCommand();
	
		try{
			db.postCommand(command);
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.Admin200703.getProfileAll.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.Admin200703.getProfileAll.error", e.getMessage());
			return exp.getMessage();
		}
		
		if(command.succeeded()){
			Document doc=Xml.newDocument();
			AbstractData data=command.getCollection();
			Element root=data.getXml(doc);
			String retXml=Xml.toString(root);
			
			LogManager.postNormal("service.Admin200703.getProfileAll.end", null);
			return retXml;
		}
		
		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.Admin200703.getProfileAll.error", "unknown");
		return exp.getMessage();
	}
	
	public String getSchema(){  // Called by children
		
		LogManager.postNormal("service.Admin200703.getSchema.begin", null);
		
		DatabaseAdapter db=DatabaseAdapter.getInstance();
		GetAdminSchemaCommand command=CommandFactory.createGetAdminSchemaCommand();
	
		try{
			db.postCommand(command);
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.Admin200703.getSchema.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.Admin200703.getSchema.error", e.getMessage());
			return exp.getMessage();
		}
		
		if(command.succeeded()){
			Document doc=Xml.newDocument();
			AbstractData data=command.getAdminSchema();
			Element root=data.getXml(doc);
			String retXml=Xml.toString(root);
			
			LogManager.postNormal("service.Admin200703.getSchema.end", null);
			return retXml;
		}
		
		// return error code
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.Admin200703.getSchema.error", "unknown");
		return exp.getMessage();
	}	
	
	public String getNeighbor(){
		
		LogManager.postNormal("service.Admin200703.getNeighbor.begin", null);
		
		DatabaseAdapter db=DatabaseAdapter.getInstance();
		GetAdminNeighborCommand command=CommandFactory.createGetAdminNeighborCommand();
	
		try{
			db.postCommand(command);
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.Admin200703.getNeighbor.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.Admin200703.getNeighbor.error", e.getMessage());
			return exp.getMessage();
		}
		
		if(command.succeeded()){
			Document doc=Xml.newDocument();
			AbstractData data=command.getAdminNeighbor();
			Element root=data.getXml(doc);

			String retXml=Xml.toString(root);
			LogManager.postNormal("service.Admin200703.getNeighbor.end", null);
			return retXml;
		}
		
		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.Admin200703.getNeighbor.error", "unknown");
		return exp.getMessage();
	}
	
	public String replicate(){
		
		LogManager.postNormal("service.Admin200703.replicate.begin", null);

		DatabaseAdapter db=DatabaseAdapter.getInstance();
		DumpAdminCommand command=CommandFactory.createDumpAdminCommand();
	
		try{
			db.postCommand(command);
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.Admin200703.replicate.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.Admin200703.replicate.error", e.getMessage());
			return exp.getMessage();
		}
		
		if(command.succeeded()){
			Document doc=Xml.newDocument();
			AbstractData data=command.getReplicatedAdmin();
			Element root=data.getXml(doc);

			String retXml=Xml.toString(root);
			LogManager.postNormal("service.Admin200703.replicate.end", null);
			return retXml;
		}
		
		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.Admin200703.replicate.error", "unknown");
		return exp.getMessage();
	}
	
	public String search(String query){
	
		LogManager.postNormal("service.Admin200703.search.begin", null);
		
		AbstractData queryData=null;
		try{
			Document doc=Xml.parseXml(query);
			Element root=doc.getDocumentElement();
		
			org.livee.data.query.ParserFactory pf=org.livee.data.query.ParserFactory.getInstance();
			AbstractParser query_parser=pf.get("query");
		
			queryData=query_parser.parse(root);

			if(queryData!=null){
				DatabaseAdapter db=DatabaseAdapter.getInstance();
				org.livee.data.query.QueryProcessCommand command=new org.livee.data.query.QueryProcessCommand(queryData);
		
				db.postCommand(command);
			
				if(command.succeeded()){
					AbstractData data=command.getQueryResponse();
					root=data.getXml(doc);
					String retXml=Xml.toString(root);
					
					LogManager.postNormal("service.Admin200703.search.end", null);
					return retXml;
				}
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.Admin200703.search.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.Admin200703.search.error", e.getMessage());
			return exp.getMessage();
		}

		// 	return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.Admin200703.search.error", "unknown");
		return exp.getMessage();
	}
	

	public String queryService(String ifName){
		
		LogManager.postNormal("service.Admin200703.queryService.begin",null);
		
		try{
			MessageContext ctx=MessageContext.getCurrentContext();
			
			String str_ActionURI=(String)ctx.getProperty("transport.url");
			java.net.URI actionURI=new java.net.URI(str_ActionURI);
						
			String scheme=actionURI.getScheme();
			String host=actionURI.getHost();
			int port=actionURI.getPort();
					
			ConfigManager config=ConfigManager.getInstance();
			String dirpath=config.getStringParameter("/c:liveeConfiguration/c:queryService/c:service/c:dirPath/text()[../../c:name/text()=\""+ifName+"\"]");
			if(dirpath!=null && !dirpath.equals("") && scheme.equals("http")){
				String url=null;
				if(port==80 || port<=0){
					url="http://"+host+dirpath;
				}else{
					url="http://"+host+":"+port+dirpath;
				}
				LogManager.postNormal("service.Admin200703.queryService.end",null);
				return url;
			}else{
				throw new NoServerResolvedLiveEException(ifName);
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.Admin200703.queryService.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.Admin200703.queryService.error", e.getMessage());
			return exp.getMessage();
		}
	}

}
