package org.livee.data.tools;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.expr.AbstractData ;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class ProfileSchemaProcessor {

	private static ProfileSchemaProcessor m_Singleton=null;
	
	long m_CacheValidityTime; // millisecond
	long m_LastCacheTime;
	AbstractData m_CachedProfileSchema;
	java.util.Map m_CachedSchemaMap;
	
	public class Schema{
		public static final int SCHEMA_TYPE_NULL=0;
		public static final int SCHEMA_TYPE_BOOLEAN=1;
		public static final int SCHEMA_TYPE_INTEGER=2;
		public static final int SCHEMA_TYPE_FLOAT=3;
		public static final int SCHEMA_TYPE_TIME=4;
		public static final int SCHEMA_TYPE_STRING=5;
		
		public String name;
		public int type;
		public String value;
		public String description;
		public String visibility;
		public boolean multilanguage;
		public String clazz;
		public String delegate;
		
		public Schema(String name,String type,String value){
			this.name=name;
			if(type.equals("boolean")){
				this.type=SCHEMA_TYPE_BOOLEAN;
			}else if(type.equals("integer")){
				this.type=SCHEMA_TYPE_INTEGER;
			}else if(type.equals("float")){
				this.type=SCHEMA_TYPE_FLOAT;
			}else if(type.equals("time")){
				this.type=SCHEMA_TYPE_TIME;
			}else if(type.equals("string")){
				this.type=SCHEMA_TYPE_STRING;
			}else{
				this.type=SCHEMA_TYPE_NULL;
			}
			this.value=value;
			this.clazz=".*";
			this.multilanguage=false;
			this.description="";
			this.visibility="";
			this.delegate=null;
		}
		
		public Schema(String name,String type,String value,String clazz,String multilanguage,String description,String visibility,String delegate){
			this.name=name;
			if(type.equals("boolean")){
				this.type=SCHEMA_TYPE_BOOLEAN;
			}else if(type.equals("integer")){
				this.type=SCHEMA_TYPE_INTEGER;
			}else if(type.equals("float")){
				this.type=SCHEMA_TYPE_FLOAT;
			}else if(type.equals("time")){
				this.type=SCHEMA_TYPE_TIME;
			}else if(type.equals("string")){
				this.type=SCHEMA_TYPE_STRING;
			}else{
				this.type=SCHEMA_TYPE_NULL;
			}
			this.value=value;
			this.clazz=clazz;
			if(multilanguage!=null && multilanguage.equals("true")){
				this.multilanguage=true;
			}else{
				this.multilanguage=false;
			}
			this.description=description;
			this.visibility=visibility;
			this.delegate=delegate;
		}
	}
	
	ProfileSchemaProcessor(){
		org.livee.data.tools.ConfigManager config=org.livee.data.tools.ConfigManager.getInstance();
		int validityTime=config.getIntParameter("/c:liveeConfiguration/c:profileSchema/c:interval/text()"); 
		if(validityTime<=0){
			m_CacheValidityTime=validityTime*1000;
		}else{
			m_CacheValidityTime=600000; // recommended default 10 min
		}
		m_CachedSchemaMap=null;
		m_Singleton=this;
	}
	
	public static ProfileSchemaProcessor getInstance(){
		if(m_Singleton==null){
			return new ProfileSchemaProcessor();
		}
		return m_Singleton;
	}
	
	public int getDataType(String name){
		long current=System.currentTimeMillis();
		if(current>(m_LastCacheTime+m_CacheValidityTime) || current<m_LastCacheTime){
			cacheRefresh();
		}
		int underbar=name.lastIndexOf('_');
		if(underbar>0){	
			name=name.substring(0,underbar);
		}
		Object obj=m_CachedSchemaMap.get(name);
		if(obj!=null){
			Schema schema=(Schema)obj;
			return schema.type;
		}
		return Schema.SCHEMA_TYPE_NULL;
	}
	
	/**
	 * 
	 * @param attrs
	 * @return error messages
	 */
	public String[] validateAttribute(AbstractData combined){
		
		java.util.ArrayList warn_messages=new java.util.ArrayList();
		
		java.util.Map attrs=combined.getAttributes();
		java.util.Iterator itr=attrs.keySet().iterator();
		
		while(itr.hasNext()){
			String key=(String)itr.next();
			if(key.equals("id")){ continue; }
			
			String value=(String)attrs.get(key);
			
			try{
				validateAttribute(key,value);
			}catch(Exception e){
				warn_messages.add(e.getMessage());
			}
		}
		
		java.util.Collection children=combined.getChildren();
		if(children!=null){
			java.util.Iterator itr2=children.iterator();
			while(itr2.hasNext()){
				AbstractData element=(AbstractData)itr2.next();
				attrs=element.getAttributes();
				itr=attrs.keySet().iterator();
				
				while(itr.hasNext()){
					String key=(String)itr.next();
					if(key.equals("id")){ continue; }		

					String value=(String)attrs.get(key);
					
					try{
						validateAttribute(key,value);
					}catch(Exception e){
						warn_messages.add(e.getMessage());
					}
				}
								
			}
		}
		
		int size=warn_messages.size();
		if(size==0){
			return null;  // No error
		}
		
		String[] warnings=new String[size];
		for(int i=0;i<size;i++){
			warnings[i]=(String)warn_messages.get(i);
		}
		return warnings;
	}
	
	/**
	 * 
	 * @param name attribute name
	 * @param value attribute value
	 * @return 
	 */
	public boolean validateAttribute(String name,String value) throws Exception {
		
		// This should be deleted...
		//if(4%2==0){
		//	return true;
		//}
		
		if(name==null){
			return false;
		}
		
		long current=System.currentTimeMillis();
		if(current>(m_LastCacheTime+m_CacheValidityTime) || current<m_LastCacheTime){
			cacheRefresh();
			m_LastCacheTime=current;
		}
		
		boolean bMultilanguage=false;
		int underbar=name.lastIndexOf('_');
		if(underbar>0){	
			name=name.substring(0,underbar);
			bMultilanguage=true;
		}
		
		Object obj=m_CachedSchemaMap.get(name);
		if(obj!=null){
			Schema schema=(Schema)obj;
			
			if(bMultilanguage && !schema.multilanguage){
				throw new Exception("\""+name+"\" does not support multilanguage.");
			}
			
			switch(schema.type){
			case (Schema.SCHEMA_TYPE_BOOLEAN):
				if(value!=null && (value.startsWith("t") || value.startsWith("f"))){
					return true;
				}else{
					throw new Exception(name+"=\""+value+"\" is invalid boolean type.");
				}
			
			case (Schema.SCHEMA_TYPE_INTEGER):
				try{
					Integer.parseInt(value);
					return true;
				}catch(Exception e){
					throw new Exception(name+"=\""+value+"\" is invalid integer type.");
				}

			case (Schema.SCHEMA_TYPE_FLOAT):
				try{
					Double.parseDouble(value);
					return true;
				}catch(Exception e){
					throw new Exception(name+"=\""+value+"\" is invalid float type.");
				}

			case (Schema.SCHEMA_TYPE_TIME):
				try{
					org.livee.util.W3CTimestamp.parse(value);
					return true;
				}catch(Exception e){
					throw new Exception(name+"=\""+value+"\" is invalid W3C timestamp type.");
				}

			case (Schema.SCHEMA_TYPE_STRING):
				if(value.matches(schema.value)){
					return true;
				}else{
					throw new Exception("\""+value+"\" is invalid \""+name+"\"");
				}

			default:
				break;
			}
		}else{
			throw new Exception("\""+name+"\" is invalid attribute name.");
		}
		
		return false;
	}
	
	public AbstractData getProfileSchema(){
		long current=System.currentTimeMillis();
		if(current>(m_LastCacheTime+m_CacheValidityTime) || current<m_LastCacheTime){
			cacheRefresh();
			m_LastCacheTime=current;
		}
		return m_CachedProfileSchema;
	}
	
	private void cacheRefresh(){
		//System.out.println("CacheRefreshed");
		try{
			org.livee.data.db.GetAdminProfileSchemaCommand command=org.livee.data.db.CommandFactory.createGetAdminProfileSchemaCommand();
			org.livee.data.db.DatabaseAdapter db=org.livee.data.db.DatabaseAdapter.getInstance();
			db.postCommand(command);
			
			if(command.succeeded()){
				org.livee.data.expr.AbstractData profileSchema=command.getProfileSchema();
				m_CachedProfileSchema=profileSchema;
				
				java.util.Map swapSchema=new java.util.Hashtable();
				java.util.Iterator itr=profileSchema.getChildren().iterator();
				while(itr.hasNext()){
					AbstractData schema=(AbstractData)itr.next();
					String name=schema.getAttribute("name");
					String type=schema.getAttribute("type");
					String value=schema.getAttribute("value");
					String clazz=schema.getAttribute("class");
					String multilanguage=schema.getAttribute("multilanguage");
					String description=schema.getAttribute("description");
					String visibility=schema.getAttribute("visibility");
					String delegate=schema.getAttribute("delegate");
					if(name!=null && type!=null){
						swapSchema.put(name,new Schema(name,type,value,clazz,multilanguage,description,visibility,delegate));
					}
				}
				
				m_CachedSchemaMap=swapSchema;
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}		
	}
	
	
}
