package org.livee.test;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.exception.LiveEException;
import org.livee.data.expr.AbstractData;
import org.livee.data.expr.DataFactory;

import org.livee.data.db.DatabaseAdapter;
import org.livee.data.db.SetProfileCommand;
import org.livee.data.db.CommandFactory;

/**
 * 
 * @author Hideya Ochiai, The University of Tokyo
 */
public class SensorProfileRegistration {

	
	private static java.util.Random m_Random=new java.util.Random();

	public static void main(String[] args) throws LiveEException {
		
		int start=0;
		int end=99;
		
		try{
			start=Integer.parseInt(args[0]);
			end=Integer.parseInt(args[1]);			
		}
		catch(Exception e){
			System.out.println("Usage: $java org.livee.test.ProfileRegistrator start end");
			// System.out.println(" Default start=0");
			// System.out.println(" Default end=100");
			System.exit(1);
		}
		System.out.println("start="+start+"; end="+end);

		
		GenerateSensorID generator=new GenerateSensorID();
		String[] combined=generator.generateCombinedID(start, end);
		
		
		for(int i=0;i<combined.length;i++){
			DataFactory df=DataFactory.getInstance();
			AbstractData combinedSensor=df.create("combined");
			
			combinedSensor.setAttribute("id", combined[i]);
			combinedSensor.setAttribute("sensorVendor", sensorVendorGenerator(i));
			combinedSensor.setAttribute("sensorModel", sensorModelGenerator(i));
			//combinedSensor.setAttribute("location_jpn", "Location Information in Japanese");
			combinedSensor.setAttribute("location_eng", "Location Information in English");
			combinedSensor.setAttribute("latitude", latitudeGenerator(i));
			combinedSensor.setAttribute("longitude", longitudeGenerator(i));
			
			String[] element=generator.generateElementID(combined[i]);
			String[] sensorType=generator.getSensorType();
			
			for(int k=0;k<element.length;k++){
				AbstractData elementSensor=df.create("element");
				elementSensor.setAttribute("id",element[k]);
				elementSensor.setAttribute("sensorType",sensorType[k]);
				combinedSensor.append(elementSensor);
			}
			
			SetProfileCommand command=CommandFactory.createSetProfileCommand(combinedSensor, "00000000");
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			
			try{
				db.postCommand(command);
				if(command.succeeded()){
					System.out.println("@"+i);
				}
			}
			catch(LiveEException e){
				throw e;
			}
			catch(Exception e){
				e.printStackTrace();
			}
			
		}	
	}
	
	private static String sensorVendorGenerator(int i){
		
		if(1==1){
			return "Vaisala";
		}
		
		switch(i%7){
		case 0:
			return "Vaisala";
		case 1:
		case 2:
		case 3:
		case 4:
		case 5:
			return "vaisala";
		case 6:
			return "VAISALA";
		default:
			return "vaisala";
		}
	}

	private static String sensorModelGenerator(int i){

		if(1==1){
			return "WXT510";
		}

		switch(i%7){
		case 0:
			return "Wxt510";
		case 1:
		case 2:
		case 3:
		case 4:
		case 5:
			return "wxt510";
		case 6:
			return "WXT510";
		default:
			return "WXT-510";
		}
	}

	private static String latitudeGenerator(int i){
		return Double.toString(m_Random.nextDouble()*180-90);
	}

	private static String longitudeGenerator(int i){
		return Double.toString(m_Random.nextDouble()*360-180);
	}
}
